**Optimizing Investments in Green Infrastructure to Mitigate Flooding in Vermont**
======================================================================


About
------------
This project aims to optimize investments in forest restoration and floodplain reconnection to mitigate flooding in Vermont. A genetic algorithm is used to identify scenarios that provide the greatest benefits, in terms of reduction in property damages from flooding, assuming cost constraints. The objective function used to estimate reductions in property damages from flooding employs HEC-RAS, a steady-flow hydrologic model developed by the Army Corps of Engineers, to estimate changes in flood extent and depth under alternative scenarios. Depth-damage functions and property values estimates are used calculate how a reduction in flood depth translate to avoided costs to property owners. 


Dependencies
------------
* deap==1.0.2
* geopandas==0.2.1
* matplotlib==2.0.0
* numpy==1.13.3+mlk
* pandas==0.20.3
* pyautogui==0.9.36
* pygeoprocessing==0.5.0
* pylint==1.7.2
* pywin32==221
* pywinauto==0.6.3
* seaborn==0.8.1
* shapefile==1.2.1


Modules
------------
* constants.py: lists basic constants, conversion coefficients, and hard codes some parameters
* driver.py: executes pre-processing functions and call ga.py
* ga.py: genetic algorithm to solve multi-objective optimization problem
* main.py: high-level script to initialize parameters and call driver.py
* make_figures.py: high-level script to call lower-level functions to create figures
* objective_function.py: evaluates fitness of scenario; executes HEC-RAS, evaluates damages from flooding
* paths.py: list of paths for input and output files
* preprocessing.py: set of functions to pre-process input data
* ras_controller.py: set of functions to automate control over HEC-RAS
* utils.py: miscellaneous utility functions

* figure_templates/
	* depth_damage.py: creates plots for depth-damage curves
	* distplot.py: creates plot for distribution of flood mitigation benefits
	* equity_weights.py: create plots showing equity-weighting functions
	* frontier.py: creates plot for tradeoff frontier
	* ws_map.py: creates map watershed restoration scenario


How to use
------------

There are two high-level scripts the user can call with python: main.py and make_figures.py.

* main.py executes the optimization algorithm and outputs results. The user may initialize parameters in main.py.
* make_figures.py executes the creation of figures based on the output data. 


Contact
------------

Please feel free to contact me with any questions, comments, or suggestions at jesse.gourevitch@uvm.edu.