import os
import time
import numpy as np
import seaborn as sb
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker

data1 = [15248.396548339933, 24727.883180127206, 41652.528997365946, 302056.60161137320]
data2 = [15846.989483998734, 24802.658488778063, 28022.982923303534, 282061.62679597037]
data3 = [15846.989483998734, 24802.658488778063, 28022.982923303534, 282061.62679597037]
data = [data1, data2, data3]

### Set plot parameters and style
sb.set(style='ticks')
fig, axes = plt.subplots(figsize=(7,5.5))

for i in range(3):

	### Plot data
	if i == 0: color, eta, symbol = 'r', i, '^'
	if i == 1: color, eta, symbol = 'k', i, 's'
	if i == 2: color, eta, symbol = 'b', i, 'o'
	x = np.array([1,  2,  3,  4])
	y = np.array(data[i])
	width = 0.25
	axes.plot(x+width*i, y, symbol, mec='k', mfc='none', mew=1,
		label=r'$\eta$ = %s' %str(eta))

### Plot gridlines
axes.plot([2-width,2-width], [-1*10**8,10**8], 'k--', alpha=0.5)
axes.plot([3-width,3-width], [-1*10**8,10**8], 'k--', alpha=0.5)
axes.plot([4-width,4-width], [-1*10**8,10**8], 'k--', alpha=0.5)

### Set the position of the x ticks
axes.set_xticks([p + width for p in x])

### Set the labels for the x ticks
axes.set_xticklabels(['First\nquartile', 'Second\nquartile', 
					  'Third\nquartile', 'Fourth\nquartile'])

### Set axis limits
axes.set_xlim(0.5+width, 5-width)
axes.set_ylim(9000, 400000)

axes.set_yscale('log')


### Set y ticks
axes.set_yticks([10000, 25000, 50000, 100000, 250000])

### Formet tick labels
fmt = '${x:,.0f}'
tick = ticker.StrMethodFormatter(fmt)
axes.yaxis.set_major_formatter(tick) 
plt.minorticks_off()

plt.tick_params(axis='x', bottom=False, pad=2)

### Set axis labels
axes.set_xlabel('Property value', labelpad=10)
axes.set_ylabel('Reduction in flood damages ($)', labelpad=10)	

### Legend
handles, labels = axes.get_legend_handles_labels()
legend = plt.legend(handles, labels, loc='upper left', frameon=True)
legend.get_frame().set_linewidth(1)
legend.get_frame().set_edgecolor('k')

### Save figure
uri = '/Users/jessegourevitch/Desktop/fig6.pdf'
plt.savefig(uri, bbox_inches='tight', transparent=True)

### Open figure
time.sleep(1)
os.system('open ' + uri)