import os
import time
import pickle
import numpy as np
import pandas as pd
import seaborn as sb
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker


### Initialize paths
stem = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
intervention_logbooks_dir = os.path.join(
	stem, 'Optimization_Outputs/intervention_logbooks')

### Read scenario list from pickle file
pickle_uri = os.path.join(stem, 'Figures/FigS6_RevisionGEC/scenario_list.pickle')
with open(pickle_uri, 'rb') as handle:
	scenario_list = pickle.load(handle)

### Get baseline data
base_csv_uri = os.path.join(intervention_logbooks_dir, 'baseline.csv')
df_base = pd.read_csv(base_csv_uri)

### Create total df
df_total = df_base.copy()

df_total['depth_ft_2yr_delta_total_ew0'] = pd.Series([0]*len(df_base))
df_total['depth_ft_10yr_delta_total_ew0'] = pd.Series([0]*len(df_base))
df_total['depth_ft_25yr_delta_total_ew0'] = pd.Series([0]*len(df_base))
df_total['depth_ft_100yr_delta_total_ew0'] = pd.Series([0]*len(df_base))
df_total['depth_ft_2yr_delta_total_ew1'] = pd.Series([0]*len(df_base))
df_total['depth_ft_10yr_delta_total_ew1'] = pd.Series([0]*len(df_base))
df_total['depth_ft_25yr_delta_total_ew1'] = pd.Series([0]*len(df_base))
df_total['depth_ft_100yr_delta_total_ew1'] = pd.Series([0]*len(df_base))
df_total['depth_ft_2yr_delta_total_ew2'] = pd.Series([0]*len(df_base))
df_total['depth_ft_10yr_delta_total_ew2'] = pd.Series([0]*len(df_base))
df_total['depth_ft_25yr_delta_total_ew2'] = pd.Series([0]*len(df_base))
df_total['depth_ft_100yr_delta_total_ew2'] = pd.Series([0]*len(df_base))

### Iterate through scenarios
for i, scenario in enumerate(scenario_list):
	scenario = map(int, scenario.values[0][1:-1].split(','))

	### Iterate through scenario interventions
	for xs, intervention in enumerate(scenario): 
		if intervention != 0:
			csv_uri = os.path.join(intervention_logbooks_dir, 
				'%s_%s.csv' %(str(xs), str(intervention)))
			df = pd.read_csv(csv_uri)

			### Get data of interest
			flood_events = [2, 10, 25, 100]
			for event in flood_events:
				depth_ft_delta = df['DEPTH_FT_%sYR_DELTA' %str(event)]
				col = 'depth_ft_%syr_delta_total_ew%s' %(str(event), str(i))
				df_total[col] += depth_ft_delta


### Subset data
df_total = df_total[df_total['PROPERTY_VALUE']>0]
df_total = df_total[df_total['DEPTH_FT_100YR']>0]

### Plot data
sb.set(style='ticks')
fig, axes = plt.subplots(4, 3, figsize=(12, 8))
plt.subplots_adjust(wspace=0.08, hspace=0.14)

df_total = df_total.reset_index()
x = df_total['PROPERTY_VALUE'] / 1000

for r in range(4):
	for k in range(len(df_total)):
		ew0 = np.array(df_total['depth_ft_%syr_delta_total_ew%s' %(str(flood_events[r]), str(0))])[k]
		ew1 = np.array(df_total['depth_ft_%syr_delta_total_ew%s' %(str(flood_events[r]), str(1))])[k]
		ew2 = np.array(df_total['depth_ft_%syr_delta_total_ew%s' %(str(flood_events[r]), str(2))])[k]

		ew0 *= 0.3048
		ew1 *= 0.3048
		ew2 *= 0.3048

		if ((abs(ew0-ew1)>0.01) or 
			(abs(ew0-ew2)>0.01) or 
			(abs(ew1-ew2)>0.01)):
			
			axes[r,0].plot(np.array(x)[k], ew0, 'ro', markersize=4, mfc='none', zorder=1000)	
			axes[r,1].plot(np.array(x)[k], ew1, 'ro', markersize=4, mfc='none', zorder=1000)	
			axes[r,2].plot(np.array(x)[k], ew2, 'ro', markersize=4, mfc='none', zorder=1000)	

			print flood_events[r]
			print k
			print ew0
			print ew1
			print ew2
			print    


for r in range(4):
	for c in range(3):
		axes[r,c].plot(range(-1, 300000), [0]*len(range(-1, 300000)), 'k--', alpha=.7)

		y1 = df_total['depth_ft_%syr_delta_total_ew%s' %(
			str(flood_events[r]), str(c))] 
		y1 *= 0.3048 # Convert feet to meters
		axes[r,c].plot(x, y1, 'ko', markersize=4, mfc='none')
		
		axes[r,c].set_xscale('symlog')

		if r==0:
			axes[r,c].set_title(r'${\eta}$ = %s' %str(c), 
				fontdict={'ha': 'center'})
		if r==3:
			axes[r,c].set_xlabel('Property Value ($)')
			fmt = '${x:,.0f}k'
			axes[r,c].set_xticks([10, 25, 50, 100, 250])
			tick = ticker.StrMethodFormatter(fmt)
			axes[r,c].xaxis.set_major_formatter(tick) 
		else:
			# axes[r,c].set_xticks([])
			axes[r,c].tick_params(
				axis='x',          # changes apply to the x-axis
				which='both',      # both major and minor ticks are affected
				bottom=False,      # ticks along the bottom edge are off
				top=False,         # ticks along the top edge are off
				labelbottom=False  # labels along the bottom edge are off)
				)
			# axes[r,c].get_xaxis().set_ticks([])

		if c==0:
			axes[r,c].text(2.5, 0.07, '%s-yr\nflood' %str(flood_events[r]),
				fontdict={'ha': 'center', 'weight': 'bold'})
			axes[r,c].set_ylabel(r'$\Delta$ Depth (m)')
		else:
			axes[r,c].set_yticks([])

		axes[r,c].set_xlim(8, 275)
		axes[r,c].set_ylim(-0.01, 0.185)

### Save figure
uri = '/Users/jessegourevitch/Desktop/s6.pdf'
plt.savefig(uri, bbox_inches='tight')

### Open figure
time.sleep(0.5)
os.system('open ' + uri)













