import os
import numpy as np
import pandas as pd

import paths, figure_templates


def execute(args):
	
	### Update args to global variables
	globals().update(args)
	output_dir = paths.figures_dir

	############################# METHODS FIGURES ##############################
	
	### Depth-damage curves
	if make_depthdamage:	
		fn = 'Methods/depthdamage_curves'
		figure_templates.depthdamage(fn, output_dir)

	### Equity-weights function - v1
	if make_equityweights_v1:	
		fn = 'Methods/equityweights_v1'
		property_values = [50000, 500000, 5000000]
		damage_costs = np.linspace(0, 10**4 + 1000, 1000)
		weights = [0, 1, 2]
		figure_templates.equityweights_v1(property_values, damage_costs, 
										  weights, fn, output_dir)

	### Equity-weights - v2
	if make_equityweights_v2:	
		fn = 'Methods/equityweights_v2'
		property_values = [10**4, 10**5, 10**6]
		damage_costs = 5000
		weights = np.linspace(-2,2,100)
		figure_templates.equityweights_v2(property_values, damage_costs, 
										  weights, fn, output_dir)


	########################## GA PERFORMANCE FIGURES ##########################

	### Get summary logbook data
	stem = paths.optimization_output_dir
	csv_uri = os.path.join(stem, 'summary_logbook.csv')
	df = pd.read_csv(csv_uri)

	### Fitness plot
	if make_fitness_plot == True:
		fn = 'GA_performance/fitness'
		figure_templates.fitness_plot(df, fn, output_dir)

	### Diversity plot
	if make_diversity_plot == True:
		fn = 'GA_performance/diversity'
		figure_templates.diversity_plot(df, fn, output_dir)

	### Violin plot
	if make_violin_plot == True:
		fn = 'GA_performance/violin'
		figure_templates.violin_plot(df, fn, output_dir)

	### KDE plot
	if make_kde_plot == True:
		fn = 'GA_performance/kde'
		figure_templates.kde_plot(df, fn, output_dir)


	######################## RESULTS FIGURES - MONETARY ########################

	### Frontier
	if make_benefitcost_frontier:
		fn = 'Results/benefitcost_frontier'
		figure_templates.benefitcost_frontier(df, fn, output_dir, 
			non_dominated=False, open_fig=True)

	### Maps
	if make_benefitcost_maps:
		### Get data
		df1 = df[df['COST']<=25000]
		df2 = df[df['COST']<=50000]
		df3 = df[df['COST']<=75000]

		b1_max = df1['AVOIDEDCOST_EW0'].max() 
		b2_max = df2['AVOIDEDCOST_EW0'].max() 
		b3_max = df3['AVOIDEDCOST_EW0'].max() 

		b1_fid = int(df1.index[df1['AVOIDEDCOST_EW0']==b1_max].max()) 
		b2_fid = int(df2.index[df2['AVOIDEDCOST_EW0']==b2_max].max()) 
		b3_fid = int(df3.index[df3['AVOIDEDCOST_EW0']==b3_max].max()) 

		scenario0 = df1.loc[df1.index  == b1_fid,  'INDIVIDUAL']
		scenario1 = df2.loc[df2.index  == b2_fid,  'INDIVIDUAL']
		scenario2 = df3.loc[df3.index  == b3_fid,  'INDIVIDUAL']
		scenario_list = [scenario0, scenario1, scenario2]

		fn = 'Results/benefitcost_maps'
		figure_templates.benefitcost_maps(scenario_list, fn, output_dir)

	### Distribution plot
	if make_benefitcost_distributionplot_baseline:
		fn = 'Results/benefitcost_distributionplot_baseline'
		figure_templates.benefitcost_distributionplot_baseline(fn, output_dir)


	######################## RESULTS FIGURES - EQUITY ##########################

	if (make_equityweighted_frontier or 
		make_equityweighted_maps or 
		make_equityweighted_distributionplots or 
		make_equityweighted_distributionbars):

		for b in [50000]:
		# for b in [10000, 20000, 50000, 100000, 150000, 
		# 		  200000, 250000, 300000, 350000, 400000]:

			for i in [2]:
			# for i in range(3):

				### Subset data to budgetary contraint
				df_copy = df.copy()
				df_copy = df_copy[df_copy['COST']<=b].reset_index()

				## Get data
				if i == 0:
					n_list = [-2, 0, 2]

					ew0_max = df_copy['AVOIDEDCOST_EW-2'].max() # Benefits (n=-2)
					ew1_max = df_copy['AVOIDEDCOST_EW0'].max() # Benefits (n=0)
					ew2_max = df_copy['AVOIDEDCOST_EW2'].max() # Benefits (n=2)

					ew0_fid = int(df_copy.index[df_copy['AVOIDEDCOST_EW-2']==ew0_max].max()) 
					ew1_fid = int(df_copy.index[df_copy['AVOIDEDCOST_EW0']==ew1_max].max()) 
					ew2_fid = int(df_copy.index[df_copy['AVOIDEDCOST_EW2']==ew2_max].max()) 

				if i == 1:
					n_list = [-1, 0, 1]

					ew0_max = df_copy['AVOIDEDCOST_EW-1'].max() # Benefits (n=-1)
					ew1_max = df_copy['AVOIDEDCOST_EW0'].max() # Benefits (n=0)
					ew2_max = df_copy['AVOIDEDCOST_EW1'].max() # Benefits (n=1)

					ew0_fid = int(df_copy.index[df_copy['AVOIDEDCOST_EW-1']==ew0_max].max()) 
					ew1_fid = int(df_copy.index[df_copy['AVOIDEDCOST_EW0']==ew1_max].max()) 
					ew2_fid = int(df_copy.index[df_copy['AVOIDEDCOST_EW1']==ew2_max].max())

				if i == 2:
					n_list = [0, 1, 2]

					ew0_max = df_copy['AVOIDEDCOST_EW0'].max() # Benefits (n=0)
					ew1_max = df_copy['AVOIDEDCOST_EW1'].max() # Benefits (n=1)
					ew2_max = df_copy['AVOIDEDCOST_EW2'].max() # Benefits (n=2)

					ew0_fid = int(df_copy.index[df_copy['AVOIDEDCOST_EW0']==ew0_max].max()) 
					ew1_fid = int(df_copy.index[df_copy['AVOIDEDCOST_EW1']==ew1_max].max()) 
					ew2_fid = int(df_copy.index[df_copy['AVOIDEDCOST_EW2']==ew2_max].max())  

				scenario0 = df_copy.loc[df_copy.index == ew0_fid, 'INDIVIDUAL']
				scenario1 = df_copy.loc[df_copy.index == ew1_fid, 'INDIVIDUAL']
				scenario2 = df_copy.loc[df_copy.index == ew2_fid, 'INDIVIDUAL']
				scenario_list = [scenario0, scenario1, scenario2]
		

				# fn = 'Results/equity_weighted_benefitcost_frontier_%s_%s' %(b, i)
				# figure_templates.ew_distributionplots(scenario_list, n_list, fn, output_dir)
				# fn = 'Results/equityweighted_distributionbars_%s_%s' %(b, i)
				# figure_templates.ew_distributionbars(scenario_list, n_list, fn, output_dir)

	### Maps
	if make_equityweighted_maps:
		fn = 'Results/equityweighted_maps'
		figure_templates.ew_maps(scenario_list, n_list, fn, output_dir)

	### Distribution plot
	if make_equityweighted_distributionplots:
		fn = 'Results/equityweighted_distributionplots'
		figure_templates.ew_distributionplots(scenario_list, n_list, fn, output_dir)
	
	if make_equityweighted_distributionbars:
		fn = 'Results/equityweighted_distributionbars'
		figure_templates.ew_distributionbars(scenario_list, n_list, fn, output_dir)

	if make_equityweighted_benefitcost_frontier:
		fn = 'Results/equity_weighted_benefitcost_frontier'
		figure_templates.equityweighted_benefitcost_fontier(df, fn, output_dir)

if __name__ == "__main__":
    main()
