import os
import json
import numpy as np
import pandas as pd

import constants, genetic_algorithm, utils

from paths import *

def execute(args):
	"""Execute multi-objective optimization.

	Parameters:
	    ws (string): watershed name
	    args (dict): dictionary containing GA parameters

	Returns:
	    None
	"""

	### Save GA args to JSON
	save_json(args)

	### Read GA inputs CSV to dataframe
	ga_df = pd.read_csv(ga_inputs_csv_uri)
	args['ga_df'] = ga_df

	### Open base outputs JSON to dict
	base_dict = json.load(open(base_json_uri))
	args['base_dict'] = base_dict

	### Create summary logbook dataframe and insert into args dict
	args['summary_logbook_df'] = pd.DataFrame()

	### Optimize!
	summary_stats = optimize(args)

	### Save GA summary logbook to CSV
	args['summary_logbook_df'].to_csv(summary_logbook_uri, index=False)

	### Print GA summary stats
	print_stats(summary_stats)

	return None


def save_json(args):
	"""Save GA arguments to JSON file.

	Parameters:

	Returns:
		None
	"""
	
	print 'Saving GA args to json file'
	with open(args_json_uri, 'w') as json_file:
	    json.dump(args, json_file, default=utils.obj2json, indent=2)

	return None


def optimize(args):
	"""Execute optimization.

	Parameters:
		args (dict): dictionary containing GA parameters

	Returns:
		summary_stats (object): GA summary statistics output 
	"""

	print 'Optimizing\n'
	summary_stats = genetic_algorithm.optimize(args)

	return summary_stats


def print_stats(summary_stats):
	"""Print summary statistics.

	Parameters:
		summary_stats (object): GA summary statistics output 

	Returns:
		None
	"""

	summary_stats.header = 'gen', 'evals', 'fitness'
	summary_stats.chapters['fitness'].header = 'min', 'mean', 'max', 'std'
	print summary_stats

	return None

