from numpy import *
from scipy import optimize
import os
from win32com.client import Dispatch
import xml.etree.ElementTree as ET
from math import sqrt


#HEC-RAS Model Class definitioin
class RASModelCal:
    def __init__(self,projloc,rastype):
        self.KillRC()
        self.project_location=projloc
        self.h_observed=[]
        self.Profiles=[]
        self.CalXS=""
        self.rastype=rastype
        self.preamble=[]
        self.epilogue=[]
        self.XS=[]
    def KillRC(self,Save=False):
        if Save:
            self.RC.Project_Save()
        os.system(r"TaskKill /F /IM ras.exe")
        self.RC=None
    def InitiateControl(self):
        if self.rastype=="4.1":
            self.RC=Dispatch("RAS41.HECRASController")
            self.RC.Project_Open(self.project_location)
        else:
            self.RC=Dispatch("RAS500.HECRASController")
            self.RC.Project_Open(self.project_location)
    def GetRiver(self):
        [nrs,rivers]=self.RC.Geometry_GetRivers()
        river=rivers[0]
        return river
    def GetReach(self):
        rivid=1
        [rivid,nrs,reaches]=self.RC.Geometry_GetReaches(rivid)
        reach=reaches[0]
        return reach
    def GetXS(self):
        if len(self.preamble)==0:
            self.GetPreamble()
            self.GetEpi()
        plan_file=self.project_location[0:len(self.project_location)-4]+'.p01'
        with open(plan_file,'w') as f:
            for line in self.preamble:
               f.write(line)
        with open(plan_file,'a') as f:
            for line in self.epilogue:
                f.write(line)
        rivid=1
        rchid=1
        profile=1
        varnum=2
        self.InitiateControl()
        self.RunRAS()
        [rivid,rchid,profile,varnum,nRS,rs,chandist,vals]=self.RC.Output_ReachOutput(rivid,rchid,profile,varnum)
        rchlen=[]
        for i in range(len(chandist)-1):
            rchlen.append(chandist[i+1]-chandist[i])
        self.ChanDist=rchlen
        xs_listraw=[]
        for xs in rs:
            xs_listraw.append(xs)
        xs_listraw.reverse()
        xs_list=[]
        for line in xs_listraw:
            raw_line=line.split()
            xs_list.append(raw_line[-1])
        self.XS=xs_list
        
    def RunRAS(self):
        if self.ShowCompWindow:
            self.RC.Compute_ShowComputationWindow()
        else:
            self.RC.Compute_HideComputationWindow()
        [retval,nmsgs,msgs]=self.RC.Compute_CurrentPlan()
        return retval
    def GetPreamble(self):
        plan_file=self.project_location[0:len(self.project_location)-4]+'.p17'
        with open(plan_file) as f:
            for line in f:
                if line.find('Node')>=0:
                    break
                self.preamble.append(line)
    def GetEpi(self):
        plan_file=self.project_location[0:len(self.project_location)-4]+'.p17'
        mark=False
        with open(plan_file) as f:
            mark=False
            for line in f:
                if line.find('Computation Interval')<0:
                    pass
                else:
                    mark=True
                if mark:
                    self.epilogue.append(line)
    
    def setEncr(self,encr_vec):
        if len(self.XS)==0:
            self.GetXS()
        rs=self.XS
        midlines=[]
        xsnum=0
        plan_file=self.project_location[0:len(self.project_location)-4]+'.p17'
        for xs in rs:
            midlines.append('Encroach Node='+xs+'\n')
            midlines.append('3       ')
            midlines.append(str(encr_vec[xsnum])[0:8]+'\n')
            xsnum=xsnum+1
            
        with open(plan_file,'w') as f:
            for line in self.preamble:
               f.write(line)

        with open(plan_file,'a') as f:
            for line in midlines:
                f.write(line)
            
        with open(plan_file,'a') as f:
            for line in self.epilogue:
                f.write(line)
    def CalculateScour(self,prof):
        bridge_node=6
        yL=self.RC.Output_NodeOutput(1,1,bridge_node,1,prof,127)[0]
        vL=self.RC.Output_NodeOutput(1,1,bridge_node,1,prof,24)[0]
        yR=self.RC.Output_NodeOutput(1,1,bridge_node,1,prof,129)[0]
        vR=self.RC.Output_NodeOutput(1,1,bridge_node,1,prof,26)[0]
        k1=0.55
        g=32.2
        FL=vL/sqrt(g*yL)
        FR=vR/sqrt(g*yR)
        YsL=yL*(4*FL**(.33)*(k1/0.55))
        YsR=yR*(4*FL**(.33)*(k1/0.55))
        retval=max([YsL,YsR])
        return(retval)
    def Get_Stations(self):
        nXS=len(self.XS)
        if nXS==0:
            self.GetXS()
            nXS=len(self.XS)
        ChanStation_R=[]
        ChanStation_L=[]
        XSStation_L=[]
        XSStation_R=[]
        for i in range(nXS+1):
            if i==6:
                pass
            else:
                retchL=self.RC.Output_NodeOutput(1,1,i+1,1,1,158)
                retchR=self.RC.Output_NodeOutput(1,1,i+1,1,1,159)
                retxsL=self.RC.Output_NodeOutput(1,1,i+1,1,1,263)
                retxsR=self.RC.Output_NodeOutput(1,1,i+1,1,1,264)
                ChanStation_R.append(retchR[0])
                ChanStation_L.append(retchL[0])
                XSStation_L.append(retxsL[0])
                XSStation_R.append(retxsR[0])
        self.stations=(ChanStation_L,ChanStation_R,XSStation_L,XSStation_R)
    def Max_EncrArea(self):
        
        retval=0
        i=0
        hl=array(self.stations[0])-array(self.stations[2])
        hr=array(self.stations[3])-array(self.stations[1])
        for l in self.ChanDist:
            retval=retval+.5*l*(hr[i]+hr[i+1]+hl[i]+hl[i+1])
            i=i+1
        return(retval)
    def Get_EncrArea(self):
        nXS=len(self.XS)
        if nXS==0:
            self.GetXS()
            nXS=len(self.XS)
        EncrStation_R=[]
        EncrStation_L=[]
        for i in range(nXS+1):
            if i==6:
                pass
            else:
                retencrL=self.RC.Output_NodeOutput(1,1,i+1,1,2,139)
                retencrR=self.RC.Output_NodeOutput(1,1,i+1,1,2,140)
                EncrStation_R.append(retencrR[0])
                EncrStation_L.append(retencrL[0])
        EncrStation_L=array(EncrStation_L)
        EncrStation_R=array(EncrStation_R)

        retval=0
        i=0
        hl=EncrStation_L-array(self.stations[2])
        hr=array(self.stations[3])-EncrStation_R
        for l in self.ChanDist:
            retval=retval+.5*l*(hr[i]+hr[i+1]+hl[i]+hl[i+1])
            i=i+1
        return(retval)
