
'''
CS352 - Flood Project
By Josh Minot Nov 27 2017; modified Feb 15 2018; further updated March 1 2018
This script takes the .csv dataframe logbook files and calculates the diversity of a generations; divieristy is 
calculated based on a pairwise evaluation of differences the values at the same loci of individuals


USE: pass the script a summar_logbook file and it will generate four figures:
    -site entropy, scenario entropy, and two phenotypic space exploration plots.

'''

import numpy as np
import os
import pandas as pd
import matplotlib.pyplot as plt
import operator
from matplotlib import cm
from scipy.stats import gaussian_kde
import scipy as sc
import seaborn as sns
import argparse

import paths


def main():
    output_df = pd.read_csv(paths.summary_logbook_uri)
    output_df = output_df[1:] #trim first row
    df_with_fixed_gens = gen_from_count(output_df)
    plot_position_ent(df_with_fixed_gens)
    plot_rest_scenario_ent(df_with_fixed_gens)
    plot_fitness_0_3(df_with_fixed_gens)

def arg_parse():
    description = 'Extract input file'
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument('-f',
                        '--file',
                        help = 'The output file of interest',
                        required = True, 
                        type = str) #or path type?

    return parser.parse_args()

def plot_position_ent(df):
    ent_list = column_entropy(df)
    plt.plot(ent_list[3], ent_list[1], linestyle = ":")
    plt.plot(ent_list[3], ent_list[0], linestyle = "-.")
    plt.plot(ent_list[3], ent_list[2], linestyle = "--")
    plt.xlabel('Generation')
    plt.title("Shannon's Entropy of Restoration Scenarios at Given Stream Position")
    plt.legend(["Max","Mean","Min"])
    plt.savefig('pos_ent_rev.png', dpi=300)
    plt.show()

def plot_rest_scenario_ent(df):
    ent_list = row_entropy(df)
    plt.plot(ent_list[3], ent_list[1], linestyle = ":")
    plt.plot(ent_list[3], ent_list[0], linestyle = "-.")
    plt.plot(ent_list[3], ent_list[2], linestyle = "--")
    plt.xlabel('Generation')
    plt.title("Shannon's Entropy of Restoration Scenarios for Entire Basin")
    plt.legend(["Max","Mean","Min"])
    plt.savefig('ind_ent.png', dpi=300)
    plt.show()

#for getting generation bins from individual's numbers
def gen_from_count(df):
    max_count = df['COUNTER'].max()
    bins = list(range(0,max_count+100,100))
    labels = list(range(0, len(bins) -1))
    print(labels)
    df['GENERATION'] = pd.cut(df['COUNTER'], bins = bins, labels = labels)
    return df

#CLEANS AND RETURNS INTS ONLY 
#for taking the column with individual arrays getting integer arrays out
def pop_cleaner(aDF):
    a = np.array(aDF['INDIVIDUAL'].str.strip('[]'))
    #print('a len {}'.format(len(a)))
    b=np.zeros( (len(a), 199) , dtype = np.int8) #may need to add a length detector
    for i in range(len(a)):
        posCount = 0
        for j in range(len(a[0])):
            try: 
                b[i][posCount]= int(a[i][j])
                posCount += 1
            except ValueError: 
                continue 
    return b


def row_entropy(aDF): #calculates the entropy of each individual row
    gens = aDF['GENERATION'].unique() # to account for missing gens
    top_ent_ave = [] # the entropy ave for each generation
    top_ent_max = []
    top_ent_min = [] 
    for gen in gens:
        tempPop = aDF.loc[aDF['GENERATION']==gen]
        cleanPop = pop_cleaner(tempPop)
        mid_ent = [] #an array of all entropy values within the generation
        for i in range(len(cleanPop)):
            unique, counts = np.unique(cleanPop[i], return_counts=True)
            p_data = counts/counts.sum()
            mid_ent.append(sc.stats.entropy(p_data, base=2)) #
        top_ent_ave.append((sum(mid_ent)/(len(mid_ent))))
        top_ent_max.append(max(mid_ent))
        top_ent_min.append(min(mid_ent))
        
    return [top_ent_ave, top_ent_max, top_ent_min, list(gens)]

def column_entropy(aDF): 
    gens = aDF['GENERATION'].unique() # to account for missing gens
    print(gens)
    top_ent_ave = [] # the entropy ave for each generation
    top_ent_max = []
    top_ent_min = [] 
    for gen in gens:
        tempPop = aDF.loc[aDF['GENERATION']==gen]
        cleanPop = pop_cleaner(tempPop)
        cleanPop = cleanPop.transpose()
        mid_ent = []
        for i in range(len(cleanPop)):
            unique, counts = np.unique(cleanPop[i], return_counts=True)
            p_data = counts/counts.sum()
            mid_ent.append(sc.stats.entropy(p_data, base=2)) #
        top_ent_ave.append((sum(mid_ent)/(len(mid_ent))))
        top_ent_max.append(max(mid_ent))
        top_ent_min.append(min(mid_ent))    
    return [top_ent_ave, top_ent_max, top_ent_min, list(gens)] #.tolist()]
        
def plot_fitness_0_3(logDF):
    g = sns.jointplot(x="FITNESS0", y="FITNESS3", data=logDF, kind="kde", color="r")
    g.plot_joint(plt.scatter, c="k", s=30, linewidth=.2, marker="+")
    g.ax_joint.collections[0].set_alpha(0)
    g.set_axis_labels("FITNESS 0 (USD)", "FITNESS 3 (USD)");
    plt.subplots_adjust(top=0.9)
    g.fig.suptitle("Phenotypic Space Exploration: \n cost of restoration as function of avoided damage")
    plt.ylim(logDF['FITNESS3'].min(),logDF['FITNESS3'].max())
    plt.savefig("Cost_Avoid.png", dpi=300)
    plt.show()

def plot_fitness_3_1(logDF):
    g = sns.jointplot(x="FITNESS1", y="FITNESS3", data=logDF, kind="kde", color="r")
    g.plot_joint(plt.scatter, c="k", s=30, linewidth=.2, marker="+")
    g.ax_joint.collections[0].set_alpha(0)
    g.set_axis_labels("FITNESS 1", "FITNESS 3 (USD)");
    plt.subplots_adjust(top=0.9)
    g.fig.suptitle("Phenotypic Space Exploration: \n cost of restoration as function of avoided damage")
    plt.ylim(logDF['FITNESS3'].min(),logDF['FITNESS3'].max())
    plt.savefig("Cost_EQI_Avoid.png", dpi=300)
    plt.show()

if __name__ == "__main__":
    main()



