import os
import time
import shutil
import win32api
import pyautogui
import pywinauto
import numpy as np
import win32com.client

from pywinauto.timings import TimeoutError
from pywinauto.application import Application
from pywinauto.win32functions import SetForegroundWindow
from pywinauto.findwindows import find_window, ElementNotFoundError

import constants

import warnings
warnings.filterwarnings("ignore")

def close_rasexe():
	"""Close HEC-RAS application.

	Parameters:
		None

	Return:
		None
	"""
	
	### If HEC-RAS is open, close it
	try:
		os.system('TASKKILL /F /IM ras.exe /T')
		time.sleep(1)
		os.system('TASKKILL /F /IM ras.exe /T')
		print 'Closing already open HEC-RAS application'

	### Otherwise, pass
	except:
		pass

	return None


def open_rasexe(ras_exe_uri):
	"""Open HEC-RAS application.

	Parameters:
		ras_exe_uri (string): path to HEC-RAS executable

	Returns:
		None 
	"""

	### Open HEC-RAS executable
	print 'Open HEC-RAS.exe'
	os.startfile(ras_exe_uri)
	
	return None


def open_rasproject(ras_basemodel_dir): 
	"""Open project in HEC-RAS.
	
	Parameters:
		ras_basemodel_dir (string): directory for HEC-RAS basemodel project

	Returns:
		rc (object): HEC-RAS controller object
	"""

	### Import RAS controller as an object handle
	rc = win32com.client.Dispatch('RAS505.HECRASCONTROLLER')

	### Open RAS project
	rasproj_uri = os.path.join(ras_basemodel_dir, 'basemodel.prj')
	rc.Project_Open(rasproj_uri)
	rc.ShowRas()

	return rc


def edit_manningsn(mn_df):
	"""Edit Manning's N table in HEC-RAS with scenario values.

	Parameters:
		mn_df (pandas dataframe): Table with base Manning's N coefficients

	Returns:
		ras_window (object): HEC-RAS window object  
	"""

	### Copy geom_df to clipboard
	mn_df.to_clipboard(excel=True, header=False, index=False, 
					   line_terminator='\r')

	### Connect to RAS application
	ras_app = Application().connect(title='HEC-RAS 5.0.5')
	ras_window = ras_app.window(title='HEC-RAS 5.0.5')
	SetForegroundWindow(find_window(title='HEC-RAS 5.0.5'))

	### Click geometry button
	ras_window['23'].click_input()
	ras_window['23'].click_input()

	### Get Geometric Data window
	x,y = win32api.GetCursorPos()
	pyautogui.moveTo(x+50, y)
	try:
		geom_title = 'Geometric Data - basegeom'
		geom_app = Application().connect(title_re=geom_title)
		geom_window = geom_app.window(title_re=geom_title)
	except:
		geom_title = 'Geometric Data'
		geom_app = Application().connect(title_re=geom_title)
		geom_window = geom_app.window(title_re=geom_title)

	### Click on Table drop-down menu
	manningsn_table_button = geom_window.MenuItem(
		"&Tables ->&Manning's n or k values (Horizontally varied)")
	manningsn_table_button.click_input()

	### Change River from Main Stem to (All Rivers)
	pyautogui.typewrite(['up'])

	### Select cells to paste over
	mannings_app = Application().connect(title_re="Edit Manning's n")
	mannings_window = mannings_app.window(title_re="Edit Manning's n")
	SetForegroundWindow(find_window(title_re="Edit Manning's n"))
	mannings_window['Set_Values...'].click_input(button='right')
	x,y = win32api.GetCursorPos()
	pyautogui.moveTo(x, y+20)
	x,y = win32api.GetCursorPos()
	pyautogui.dragTo(x+1600, y, 1, button='left')

	### Past mn_df saved to clipboard
	mannings_window['0'].click_input()

	### Click 'OK'
	mannings_window['OK'].click_input()

	### Exit out of geomtry window
	geom_window.close()

	return ras_window


def open_editedterrain(ras_window, dem_uri):
	"""Upload edited terrain layer to HEC-RAS.

	Parameters:
		ras_window (object): HEC-RAS window object  
		dem_uri (str): path to edited DEM GeoTIFF layer

	Returns:
		None 
	"""

	### Open and connect to RAS Mapper window
	ras_window['12'].click_input()
	
	### Remove existing terrains
	processing = True
	while processing:
		try:
			mapper_app = Application().connect(title_re='RAS Mapper')
			mapper_window = mapper_app.window(title_re='RAS Mapper')
			SetForegroundWindow(find_window(title_re='RAS Mapper'))
			treeview = mapper_window['TreeView']
			terrain_button = treeview.get_item(['Terrains', 'Terrain'])
			terrain_button.click_input(button='right')			
			x,y = win32api.GetCursorPos()
			pyautogui.click(x, y+70)
			processing = False
		except:
			pass
	
	### Delete existing terrain directory
	terrain_dir = os.path.join(os.path.dirname(dem_uri), 'Terrain')
	if os.path.isdir(terrain_dir):
		shutil.rmtree(terrain_dir)
	
	### Open create new terrain window
	terrain_button = treeview.get_item(['Terrains'])
	terrain_button.click_input(button='right')
	x,y = win32api.GetCursorPos()
	pyautogui.click(x, y+2)
	
	### Connect to New Terrain Layer window and open Browse window
	newterrain_app = Application().connect(title_re='New Terrain Layer')
	newterrain_window = newterrain_app.window(title_re='New Terrain Layer')
	SetForegroundWindow(find_window(title_re='New Terrain Layer'))
	newterrain_window['Button6'].click_input()
	
	### Connect to Browse window and type in new terrain fn
	### Get Geometric Data window
	processing = True
	while processing:
		try:
			try:
				browseterrain_title = 'Browse for Terrain raster files'
				browse_app = Application().connect(title_re=browseterrain_title)
				browse_window = browse_app.window(title_re=browseterrain_title)
			except:
				browseterrain_title = 'Browse for Terrain'
				browse_app = Application().connect(title_re=browseterrain_title)
				browse_window = browse_app.window(title_re=browseterrain_title)
			processing = False
		except:
			pass

	pyautogui.typewrite('edited_dem.tif')
	browse_window['&Open'].click_input()

	### Set RAS SRS
	pyautogui.typewrite('enter')

	### Click create new terrain
	processing = True
	while processing:
		try:
			SetForegroundWindow(find_window(title_re='New Terrain Layer'))
			newterrain_window['Create'].click_input()
			create_terrain = "Creating Terrain 'Terrain'"
			createterrain_app = Application().connect(title_re=create_terrain)
			createterrain_window = createterrain_app.window(title_re=create_terrain)
			processing = False
		except:
			pass
	
	processing = True
	while processing:
	    try:
	        createterrain_window['Close'].click_input()
	        processing = False
	    except:
	        pass
	mapper_window.close()

	return None


def edit_cutprofiles(ras_window):
	"""Edit cross-section cut profiles to reflected edited terrain layer.

	Parameters:
		ras_window (object): HEC-RAS window object  

	Returns:
		geom_window (object): HEC-RAS geometry window object 
	"""

	### Click geometry button
	ras_window['23'].click_input()

	### Get Geometric Data window
	try:
		geom_title = 'Geometric Data - basegeom'
		geom_app = Application().connect(title_re=geom_title)
		geom_window = geom_app.window(title_re=geom_title)
	except:
		geom_title = 'Geometric Data'
		geom_app = Application().connect(title_re=geom_title)
		geom_window = geom_app.window(title_re=geom_title)

	### Click on Table drop-down menu
	cutprofiles_button = geom_window.MenuItem(
		"&GIS Tools->Terrain ...->Cut Profiles (XS, BR, Culv, IW, LW)")
	cutprofiles_button.click_input()

	### Associate all cross-sections with new terrain
	cutprofiles_app = Application().connect(
		title="Select XS's to cut profiles from the associated terrain")
	cutprofiles_window = cutprofiles_app.window(
		title="Select XS's to cut profiles from the associated terrain")
	cutprofiles_window['1'].click_input()
	cutprofiles_window['OK'].click_input()

	### Close Geometry window
	processing = True
	while processing:
		try:
			terraincutting_app = Application().connect(
				title="Terrain Cutting Profiles")
			terraincutting_window = terraincutting_app.window(
				title="Terrain Cutting Profiles")
			terraincutting_window.close()
			processing = False
		except:
			pass

	return geom_window


def filter_xspoints(ras_window):
	"""Filter cross-section points after cut profiles have been edited to 
		reflect edited terrain layer. For whatever reason, after cut profiles 
		have been edited, some cross sections have too many points (>500) and 
		the model crashes. Need to filter out some of these points so model 
		runs successfully.

	Parameters:
		geom_window (object): HEC-RAS geometry window object

	Returns:
		None
	"""

	### Click geometry button
	ras_window['23'].click_input()

	### Get Geometric Data window
	try:
		geom_title = 'Geometric Data - basegeom'
		geom_app = Application().connect(title_re=geom_title)
		geom_window = geom_app.window(title_re=geom_title)
	except:
		geom_title = 'Geometric Data'
		geom_app = Application().connect(title_re=geom_title)
		geom_window = geom_app.window(title_re=geom_title)

	### Open Cross Section Points Filter from "Tools" drop-down menu
	filter_xspoints_button = geom_window.MenuItem(
		"&Tools->Cross Section Points Filter ...")
	filter_xspoints_button.click_input()

	### Connect to Cross Section Points Filter window
	filter_xspoints_app = Application().connect(
		title="Cross Section Point Filter")
	filter_xspoints_window = filter_xspoints_app.window(
		title="Cross Section Point Filter")

	### Only filter corss sections with more than 500 points
	SetForegroundWindow(find_window(title_re='Cross Section Point Filter'))
	filter_button = 'Only filter cross sections with more than 500 points'
	filter_xspoints_window[filter_button].click_input()

	### Go to 'Multiple Locations' tab
	x,y = win32api.GetCursorPos()
	pyautogui.click(x-93, y-307)

	### Reconnect to Cross Section Points Filter window
	filter_xspoints_app = Application().connect(
		title="Cross Section Point Filter")
	filter_xspoints_window = filter_xspoints_app.window(
		title="Cross Section Point Filter")

	### Select Locations
	SetForegroundWindow(find_window(title_re='Cross Section Point Filter'))
	x,y = win32api.GetCursorPos()
	pyautogui.click(x+61, y+38)
	pyautogui.typewrite(['up', 'enter'])
	x,y = win32api.GetCursorPos()
	pyautogui.click(x-3, y+77)

	### Go to 'Minimize Area Change' tab
	x,y = win32api.GetCursorPos()
	pyautogui.click(x-2, y+174)

	### Reconnect to Cross Section Points Filter window
	filter_xspoints_app = Application().connect(
		title="Cross Section Point Filter")
	filter_xspoints_window = filter_xspoints_app.window(
		title="Cross Section Point Filter")
	
	### Filter points
	x,y = win32api.GetCursorPos()
	pyautogui.click(x+66, y+34)
	pyautogui.typewrite('485')
	x,y = win32api.GetCursorPos()
	pyautogui.click(x+68, y-2)

	### Close windows
	processing = True
	while processing:
		try:
			hecras_app = Application().connect(title="HEC-RAS")
			hecras_window = filter_xspoints_app.window(title="HEC-RAS")
			hecras_window.close()
			processing = False
		except:
			pass

	filter_xspoints_window.close()
	geom_window.close()

	return None


def change_steadyflow(ras_window, flood_event, pk_series):
	"""Change flow data in HEC_RAS to specified flood event.

	Parameters:
		ras_window (object): HEC-RAS window object
		flood_event (int): frequency of flood event occurance
		pk_series (pandas series): peak flow data for flood event

	Returns:
		None
	"""

	### Click flow button
	ras_window['22'].click_input()

	### Copy geom_df to clipboard
	pk_series.to_frame().to_clipboard(excel=True, header=False, index=False, 
									  line_terminator='\r')

	### Get Flow Data window
	flow_title = 'Steady Flow Data'
	flow_app = Application().connect(title_re=flow_title)
	flow_window = flow_app.window(title_re=flow_title)

	### Click on PF column in flow window
	SetForegroundWindow(find_window(title_re=flow_title))
	flow_window['Add A Flow Change Location'].click_input(button='right')
	x,y = win32api.GetCursorPos()
	pyautogui.click(x, y+50)

	### Paste values into PF column in flow window
	pyautogui.hotkey('ctrl', 'v') 

	### Exit out of flow window
	flow_window.close()

	return None


def compute_steadyflow(ras_window, scenario_id):
	"""Compute steady-flow analysis.

	Parameters:
		ras_window (object): HEC-RAS window object  
		scenario_id (int): ID value for function evaluation

	Returns:
		None
	"""

	### Open steady-flow analysis from Run drop-down menu
	ras_window.MenuItem("Run ->&Steady Flow Analysis...").click_input()
	steadyflow_app = Application().connect(title_re='Steady Flow Analysis')
	steadyflow_window = steadyflow_app.window(title_re='Steady Flow Analysis')
	SetForegroundWindow(find_window(title_re='Steady Flow Analysis'))

	### Edit Short ID field
	steadyflow_window['Edit2'].double_click()
	pyautogui.typewrite(['backspace'])
	pyautogui.typewrite(list(str(scenario_id)))

	### Click compute
	steadyflow_window['Compute'].click_input()

	### Close HEC-RAS Finished Computations windows
	computations_window = steadyflow_app.window(
		title_re=r'HEC-RAS Finished Computations')
	processing = True
	while processing:
	    try:
	        computations_window.close()
	        processing = False
	    except (ElementNotFoundError, TimeoutError):
	        pass

	### Close Steady Flow Analyis Window
	steadyflow_window.close()

	return None

